#!/usr/bin/env python
"""Convert .s songs to .u Unicode or to .htm
In directory hindi/a for example:
../../s_to_utf.py -w -t ../../pageUTF8.tmpl 50.s  51.s 52.s
will create xht/50.utf8 and xht/50.htm files
"""

import getopt
import os
import os.path
import subprocess
import sys

REPLACE_WORDS = {
        "\\threedots" : "##...##",
        "\\-" : "-",
        # "\\startsong\n" : '', # replace with custom string - name of song
        "\\endsong\n" : '',
        # "\\printtitle\n" : '<div class="webitrans-indian">',
        # "\\printtitle \n" : '<div class="webitrans-indian">', # edit all .s, remove trailing space
        }

def usage():                         
    print "Usage"
    print "In directory hindi/a for example:"
    print "../../s_to_utf.py -w -t ../../pageUTF8.tmpl 50.s  51.s 52.s"
    print "OR: ../../s_to_utf.py -w -t ../../pageUTF8.tmpl *.s"
    print "will create xht/50.htm files"
    print "Use -u if also need a .xht/50.utf8 file (usually not needed)"

def to_utf8(name):                         
    """Convert the single file name .s song files to .u Unicode by running ITRANS"""
    song = ""
    with open(name, "r") as fin:
        song = "<!-- #output=UTF_8 #indianifm=udvng.ifm -->\n" + fin.read()

    # preprocess the song input
    for key, val in REPLACE_WORDS.iteritems():
        song = song.replace(key, val)
    # \stitle may contain ## shortmarkers, so disable them otherwise itrans will convert
    # parts of some stitles (wrongly) into Devanagari. The goal is to not convert any of
    # stitle into Devanagari - even the parts that are in itrans encoding.
    song = song.replace("\\startsong", '#ignoreshortmarkers% ' + name + ' isongs output')
    song = song.replace("\\printtitle", '#useshortmarkers<span class="webitrans-indian">')

    p = subprocess.Popen(["itrans", "-U"],
            stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = p.communicate(song)
    rc = p.returncode

    uout = '  <div class="webitrans-output"><pre>' + out + '  </span></pre></div>'
    if rc != 0:
        print STDERR, "Failed: ", name, err, " Error return: ", rc
        return ""

    return uout

def all_to_utf8(args):                         
    """Convert the list args .s song file names to .utf8 Unicode by running ITRANS"""
    # Unused - don't keep just the unicode part around if keeping the .htm file around too,
    # since it duplicates a large portion of the file data, and nearly doubles space requirements.
    for name in args:
        song = ""
        uout = to_utf8(name)

        # output the itrans stdout
        outname = os.path.basename(name)
        suffix = outname.rfind(".s")
        if suffix > 0:
            outname = outname[:suffix]

        # Write UTF8 song file to current dir named "xht" with suffix .utf8
        with open("xht/" + outname + ".utf8", "w") as fout:
            fout.write(uout)


def all_to_html(tmpl, args):                         
    """Convert the list args .s song files to .htm Unicode disk files"""

    # Read in the HTML template
    html_tmpl = ""
    with open(tmpl, "r") as fin:
        html_tmpl = fin.read()

    # Replace WEBITRANS_UTF8_TEXTHERE with song UTF8 text
    code = "WEBITRANS_UTF8_TEXTHERE"
    pos_code = html_tmpl.find(code)
    html_pre = html_tmpl[:pos_code]
    html_post = html_tmpl[pos_code + len(code):]

    for name in args:
        outname = os.path.basename(name)
        suffix = outname.rfind(".s")
        if suffix > 0:
            outname = outname[:suffix]
        uout = to_utf8(name)

        # Write HTML song file to current dir named "xht" with suffix .htm
        with open("xht/" + outname + ".htm", "w") as fout:
            fout.write(html_pre.replace('WEBITRANS_SONGNAME', name))
            fout.write(uout)
            fout.write(html_post)


def main(argv):                         
    try:                                
        opts, args = getopt.getopt(argv, "huwt:", ["help", "utf8", "html", "template="])
    except getopt.GetoptError:
        usage()
        sys.exit(2)                     

    html_tmpl = "pageUTF8-i.tmpl"
    to_utf8 = False
    to_html = False
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit()                     
        elif opt == "-t":
            html_tmpl = arg
        elif opt == "-u":
            to_utf8 = True
        elif opt == "-w":
            to_html = True
        elif opt == "-g":
            sys.exit(100)                     
        else:
            assert False, "Unhandled option"

    if to_utf8:
        all_to_utf8(args)

    if to_html:
        all_to_html(html_tmpl, args)

if __name__ == "__main__":
    main(sys.argv[1:])
